@@ -21,6 +21,7 @@
 #define G_LOG_DOMAIN "GcalCalendarMonitor"
 
 #include "gcal-calendar-monitor.h"
+#include "gcal-core-macros.h"
 #include "gcal-date-time-utils.h"
 #include "gcal-debug.h"
 #include "gcal-event.h"
@@ -31,8 +32,8 @@
 typedef struct
 {
   GcalCalendarMonitor *monitor;
-  GcalEvent           *event;
-  gchar               *event_id;
+  GPtrArray           *events;
+  GPtrArray           *event_ids;
   gboolean             complete;
 } IdleData;
 
@@ -59,13 +60,16 @@ struct _GcalCalendarMonitor
   GcalCalendar       *calendar;
   gboolean            complete;
 
+  const GcalCalendarMonitorListener *listener;
+  gpointer                           listener_user_data;
+
   /*
    * These fields are only accessed on the monitor thread, and
    * never on the main thread.
    */
   struct {
     gboolean          populated;
-    GHashTable       *events_to_add;
+    GPtrArray        *events_to_add;
     ECalClientView   *view;
   } monitor_thread;
 
@@ -81,21 +85,13 @@ struct _GcalCalendarMonitor
   } shared;
 };
 
-static gboolean      add_event_to_timeline_in_idle_cb            (gpointer           user_data);
-static gboolean      update_event_in_idle_cb                     (gpointer           user_data);
-static gboolean      remove_event_from_timeline_in_idle_cb       (gpointer           user_data);
+static gboolean      add_events_to_timeline_in_idle_cb           (gpointer           user_data);
+static gboolean      update_events_in_idle_cb                    (gpointer           user_data);
+static gboolean      remove_events_from_timeline_in_idle_cb      (gpointer           user_data);
 static gboolean      complete_in_idle_cb                         (gpointer           user_data);
 
 G_DEFINE_TYPE (GcalCalendarMonitor, gcal_calendar_monitor, G_TYPE_OBJECT)
 
-enum
-{
-  EVENT_ADDED,
-  EVENT_UPDATED,
-  EVENT_REMOVED,
-  N_SIGNALS,
-};
-
 enum
 {
   PROP_0,
@@ -104,7 +100,6 @@ enum
   N_PROPS
 };
 
-static gulong signals[N_SIGNALS] = { 0, };
 static GParamSpec *properties [N_PROPS] = { NULL, };
 
 /*
@@ -164,12 +159,7 @@ maybe_init_event_arrays (GcalCalendarMonitor *self)
     return;
 
   if (!self->monitor_thread.events_to_add)
-    {
-      self->monitor_thread.events_to_add = g_hash_table_new_full (g_str_hash,
-                                                                  g_str_equal,
-                                                                  g_free,
-                                                                  g_object_unref);
-    }
+    self->monitor_thread.events_to_add = g_ptr_array_new_with_free_func (g_object_unref);
 }
 
 static GcalRange*
@@ -188,58 +178,64 @@ static void
 idle_data_free (IdleData *data)
 {
   g_clear_object (&data->monitor);
-  g_clear_object (&data->event);
-  g_clear_pointer (&data->event_id, g_free);
+  g_clear_pointer (&data->events, g_ptr_array_unref);
+  g_clear_pointer (&data->event_ids, g_ptr_array_unref);
   g_free (data);
 }
 
 static void
-add_event_in_idle (GcalCalendarMonitor *self,
-                   GcalEvent        *event)
+add_events_in_idle (GcalCalendarMonitor *self,
+                    GPtrArray           *events)
 {
   IdleData *idle_data;
 
+  g_assert (GCAL_IS_THREAD (self->thread));
+
   idle_data = g_new0 (IdleData, 1);
   idle_data->monitor = g_object_ref (self);
-  idle_data->event = g_object_ref (event);
+  idle_data->events = g_ptr_array_ref (events);
 
   g_main_context_invoke_full (self->main_context,
                               G_PRIORITY_DEFAULT_IDLE,
-                              add_event_to_timeline_in_idle_cb,
+                              add_events_to_timeline_in_idle_cb,
                               idle_data,
                               (GDestroyNotify) idle_data_free);
 }
 
 static void
-update_event_in_idle (GcalCalendarMonitor *self,
-                      GcalEvent           *event)
+update_events_in_idle (GcalCalendarMonitor *self,
+                       GPtrArray           *events)
 {
   IdleData *idle_data;
 
+  g_assert (GCAL_IS_THREAD (self->thread));
+
   idle_data = g_new0 (IdleData, 1);
   idle_data->monitor = g_object_ref (self);
-  idle_data->event = g_object_ref (event);
+  idle_data->events = g_ptr_array_ref (events);
 
   g_main_context_invoke_full (self->main_context,
                               G_PRIORITY_DEFAULT_IDLE,
-                              update_event_in_idle_cb,
+                              update_events_in_idle_cb,
                               idle_data,
                               (GDestroyNotify) idle_data_free);
 }
 
 static void
-remove_event_in_idle (GcalCalendarMonitor *self,
-                      const gchar         *event_id)
+remove_events_in_idle (GcalCalendarMonitor *self,
+                       GPtrArray           *events)
 {
   IdleData *idle_data;
 
+  g_assert (GCAL_IS_THREAD (self->thread));
+
   idle_data = g_new0 (IdleData, 1);
   idle_data->monitor = g_object_ref (self);
-  idle_data->event_id = g_strdup (event_id);
+  idle_data->event_ids = g_ptr_array_ref (events);
 
   g_main_context_invoke_full (self->main_context,
                               G_PRIORITY_DEFAULT_IDLE,
-                              remove_event_from_timeline_in_idle_cb,
+                              remove_events_from_timeline_in_idle_cb,
                               idle_data,
                               (GDestroyNotify) idle_data_free);
 }
@@ -250,6 +246,8 @@ set_complete_in_idle (GcalCalendarMonitor *self,
 {
   IdleData *idle_data;
 
+  g_assert (GCAL_IS_THREAD (self->thread));
+
   idle_data = g_new0 (IdleData, 1);
   idle_data->monitor = g_object_ref (self);
   idle_data->complete = complete;
@@ -284,6 +282,8 @@ client_instance_generated_cb (ICalComponent  *icomponent,
   data = user_data;
   self = data->monitor;
 
+  g_assert (GCAL_IS_THREAD (self->thread));
+
   if (g_cancellable_set_error_if_cancelled (cancellable, error))
     return FALSE;
 
@@ -310,16 +310,20 @@ on_client_view_objects_added_cb (ECalClientView      *view,
                                  GcalCalendarMonitor *self)
 {
   g_autoptr (GPtrArray) components_to_expand = NULL;
+  g_autoptr (GPtrArray) events_to_add = NULL;
   g_autoptr (GcalRange) range = NULL;
   const GSList *l;
   gint i;
 
   GCAL_ENTRY;
 
+  g_assert (GCAL_IS_THREAD (self->thread));
+
   range = get_monitor_ranges (self);
 
   maybe_init_event_arrays (self);
   components_to_expand = g_ptr_array_new ();
+  events_to_add = g_ptr_array_new_with_free_func (g_object_unref);
 
   for (l = objects; l; l = l->next)
     {
@@ -362,25 +366,14 @@ on_client_view_objects_added_cb (ECalClientView      *view,
         }
 
       if (!self->monitor_thread.populated)
-        {
-          g_autofree gchar *event_id = NULL;
-
-          event_id = g_strdup (gcal_event_get_uid (event));
-
-          g_hash_table_insert (self->monitor_thread.events_to_add,
-                               g_steal_pointer (&event_id),
-                               g_object_ref (event));
-        }
+        g_ptr_array_add (self->monitor_thread.events_to_add, g_object_ref (event));
       else
-        {
-          add_event_in_idle (self, event);
-        }
+        g_ptr_array_add (events_to_add, g_object_ref (event));
     }
 
   /* Recurrent events */
   if (components_to_expand->len > 0)
     {
-      g_autoptr (GPtrArray) expanded_events = NULL;
       g_autoptr (GDateTime) range_start = NULL;
       g_autoptr (GDateTime) range_end = NULL;
       ECalClient *client;
@@ -396,8 +389,6 @@ on_client_view_objects_added_cb (ECalClientView      *view,
       range_start_time = g_date_time_to_unix (range_start);
       range_end_time = g_date_time_to_unix (range_end) - 1;
 
-      expanded_events = g_ptr_array_new_with_free_func (g_object_unref);
-
       /* Generate the instances */
       for (i = 0; i < components_to_expand->len; i++)
         {
@@ -413,10 +404,14 @@ on_client_view_objects_added_cb (ECalClientView      *view,
           icomponent = g_ptr_array_index (components_to_expand, i);
 
           recurrences_data.monitor = self;
-          recurrences_data.expanded_events = expanded_events;
+
+          if (!self->monitor_thread.populated)
+            recurrences_data.expanded_events = self->monitor_thread.events_to_add;
+          else
+            recurrences_data.expanded_events = events_to_add;
 
 #if GCAL_ENABLE_TRACE
-          old_size = expanded_events->len;
+          old_size = recurrences_data.expanded_events->len;
 #endif
 
           e_cal_client_generate_instances_for_object_sync (client,
@@ -434,37 +429,16 @@ on_client_view_objects_added_cb (ECalClientView      *view,
               GCAL_TRACE_MSG ("Component %s (%s) added %d instance(s) between %s",
                               i_cal_component_get_summary (icomponent),
                               i_cal_component_get_uid (icomponent),
-                              expanded_events->len - old_size,
+                              recurrences_data.expanded_events->len - old_size,
                               range_str);
             }
 #endif
         }
-
-      /* Process all these instances */
-      for (i = 0; i < expanded_events->len; i++)
-        {
-          GcalEvent *event = g_ptr_array_index (expanded_events, i);
-
-          if (g_cancellable_is_cancelled (self->cancellable))
-            return;
-
-          if (!self->monitor_thread.populated)
-            {
-              g_autofree gchar *event_id = NULL;
-
-              event_id = g_strdup (gcal_event_get_uid (event));
-
-              g_hash_table_insert (self->monitor_thread.events_to_add,
-                                   g_steal_pointer (&event_id),
-                                   g_object_ref (event));
-            }
-          else
-            {
-              add_event_in_idle (self, event);
-            }
-        }
     }
 
+  if (events_to_add->len > 0)
+    add_events_in_idle (self, events_to_add);
+
   GCAL_EXIT;
 }
 
@@ -476,22 +450,25 @@ on_client_view_objects_modified_cb (ECalClientView      *view,
   g_autoptr (GRWLockReaderLocker) reader_locker = NULL;
   g_autoptr (GHashTable) events_to_remove = NULL;
   g_autoptr (GPtrArray) components_to_expand = NULL;
+  g_autoptr (GPtrArray) event_ids_to_remove = NULL;
+  g_autoptr (GPtrArray) events_to_update = NULL;
   g_autoptr (GcalRange) range = NULL;
-  GHashTableIter iter;
   const GSList *l;
-  const gchar *aux;
 
   GCAL_ENTRY;
 
+  g_assert (GCAL_IS_THREAD (self->thread));
+
   if (!self->monitor_thread.populated && self->monitor_thread.events_to_add)
     {
-      g_clear_pointer (&self->monitor_thread.events_to_add, g_hash_table_destroy);
+      g_clear_pointer (&self->monitor_thread.events_to_add, g_ptr_array_unref);
       return;
     }
 
   reader_locker = g_rw_lock_reader_locker_new (&self->shared.lock);
   components_to_expand = g_ptr_array_new ();
   events_to_remove = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+  events_to_update = g_ptr_array_new_with_free_func (g_object_unref);
   range = gcal_range_copy (self->shared.range);
 
   for (l = objects; l; l = l->next)
@@ -531,6 +508,9 @@ on_client_view_objects_modified_cb (ECalClientView      *view,
 
       if (!e_cal_component_id_get_rid (component_id))
         {
+          GHashTableIter iter;
+          const gchar *aux;
+
           g_hash_table_iter_init (&iter, self->shared.events);
           while (g_hash_table_iter_next (&iter, (gpointer*) &aux, NULL))
             {
@@ -562,13 +542,14 @@ on_client_view_objects_modified_cb (ECalClientView      *view,
           continue;
         }
 
-      update_event_in_idle (self, event);
+      g_ptr_array_add (events_to_update, g_steal_pointer (&event));
     }
 
   /* Recurrent events */
   if (components_to_expand->len > 0)
     {
       g_autoptr (GPtrArray) expanded_events = NULL;
+      g_autoptr (GPtrArray) events_to_add = NULL;
       g_autoptr (GDateTime) range_start = NULL;
       g_autoptr (GDateTime) range_end = NULL;
       ECalClient *client;
@@ -585,6 +566,7 @@ on_client_view_objects_modified_cb (ECalClientView      *view,
       range_end_time = g_date_time_to_unix (range_end) - 1;
 
       expanded_events = g_ptr_array_new_with_free_func (g_object_unref);
+      events_to_add = g_ptr_array_new_with_free_func (g_object_unref);
 
       /* Generate the instances */
       for (guint i = 0; i < components_to_expand->len; i++)
@@ -639,20 +621,26 @@ on_client_view_objects_modified_cb (ECalClientView      *view,
           old_event = g_hash_table_lookup (self->shared.events, gcal_event_get_uid (event));
           if (old_event)
             {
-              update_event_in_idle (self, event);
               g_hash_table_remove (events_to_remove, gcal_event_get_uid (event));
+              g_ptr_array_add (events_to_update, g_object_ref (event));
             }
           else
             {
-              add_event_in_idle (self, event);
+              g_ptr_array_add (events_to_add, g_object_ref (event));
             }
         }
+
+      if (events_to_add->len > 0)
+        add_events_in_idle (self, events_to_add);
     }
 
+  if (events_to_update->len > 0)
+    update_events_in_idle (self, events_to_update);
+
   /* Now remove lingering events */
-  g_hash_table_iter_init (&iter, events_to_remove);
-  while (g_hash_table_iter_next (&iter, (gpointer*) &aux, NULL))
-    remove_event_in_idle (self, aux);
+  event_ids_to_remove = g_hash_table_steal_all_keys (events_to_remove);
+  if (event_ids_to_remove->len > 0)
+    remove_events_in_idle (self, event_ids_to_remove);
 
   GCAL_EXIT;
 }
@@ -662,10 +650,15 @@ on_client_view_objects_removed_cb (ECalClientView      *view,
                                    const GSList        *objects,
                                    GcalCalendarMonitor *self)
 {
+  g_autoptr (GPtrArray) event_ids = NULL;
   const GSList *l;
 
   GCAL_ENTRY;
 
+  g_assert (GCAL_IS_THREAD (self->thread));
+
+  event_ids = g_ptr_array_new_with_free_func (g_free);
+
   for (l = objects; l; l = l->next)
     {
       g_autofree gchar *event_id = NULL;
@@ -703,13 +696,16 @@ on_client_view_objects_removed_cb (ECalClientView      *view,
           while (g_hash_table_iter_next (&iter, (gpointer*) &aux, NULL))
             {
               if (!g_str_equal (aux, event_id) && g_str_has_prefix (aux, event_id))
-                remove_event_in_idle (self, aux);
+                g_ptr_array_add (event_ids, g_strdup (aux));
             }
         }
 
-      remove_event_in_idle (self, event_id);
+      g_ptr_array_add (event_ids, g_strdup (event_id));
     }
 
+  if (event_ids->len > 0)
+    remove_events_in_idle (self, event_ids);
+
   GCAL_EXIT;
 }
 
@@ -718,33 +714,17 @@ on_client_view_complete_cb (ECalClientView      *view,
                             const GError        *error,
                             GcalCalendarMonitor *self)
 {
-  g_autoptr (GHashTable) events_to_add = NULL;
+  g_autoptr (GPtrArray) events_to_add = NULL;
 
   GCAL_ENTRY;
 
+  g_assert (GCAL_IS_THREAD (self->thread));
   g_assert (!self->monitor_thread.populated);
 
   events_to_add = g_steal_pointer (&self->monitor_thread.events_to_add);
 
-  /* Process all these instances */
   if (events_to_add)
-    {
-      GHashTableIter iter;
-      GcalEvent *event;
-
-      g_hash_table_iter_init (&iter, events_to_add);
-      while (g_hash_table_iter_next (&iter, NULL, (gpointer*) &event))
-        {
-          if (g_cancellable_is_cancelled (self->cancellable))
-            return;
-
-          GCAL_TRACE_MSG ("Sending event '%s' (%s) to main thead's idle",
-                          gcal_event_get_summary (event),
-                          gcal_event_get_uid (event));
-
-          add_event_in_idle (self, event);
-        }
-    }
+    add_events_in_idle (self, events_to_add);
 
   self->monitor_thread.populated = TRUE;
 
@@ -766,6 +746,8 @@ create_view (GcalCalendarMonitor *self)
 
   GCAL_ENTRY;
 
+  g_assert (GCAL_IS_THREAD (self->thread));
+
   reader_locker = g_rw_lock_reader_locker_new (&self->shared.lock);
 
   g_assert (self->cancellable == NULL);
@@ -826,6 +808,8 @@ remove_view (GcalCalendarMonitor *self)
 
   GCAL_ENTRY;
 
+  g_assert (GCAL_IS_THREAD (self->thread));
+
   g_clear_object (&self->cancellable);
 
   if (!self->monitor_thread.view)
@@ -864,6 +848,15 @@ message_queue_source_prepare (GSource *source,
   queue_source = (MessageQueueSource*) source;
   self = queue_source->monitor;
 
+  /*
+   * The prepare function can be called before g_thread_new()
+   * returns, and that's valid. However, in that case, no messages
+   * should be available in the queue, as the main thread didn't
+   * return execution to the mainloop yet. Protect against that.
+   */
+  g_assert (GCAL_IS_THREAD (self->thread) ||
+            (!self->thread && g_async_queue_length (self->messages) == 0));
+
   return g_async_queue_length (self->messages) > 0;
 }
 
@@ -882,6 +875,8 @@ message_queue_source_dispatch (GSource     *source,
   self = queue_source->monitor;
   event = GPOINTER_TO_INT (g_async_queue_pop (self->messages));
 
+  g_assert (GCAL_IS_THREAD (self->thread));
+
   switch (event)
     {
     case FILTER_UPDATED:
@@ -951,7 +946,6 @@ calendar_view_thread_func (gpointer data)
   g_source_attach ((GSource*) queue_source, self->thread_context);
 
   g_main_loop_run (mainloop);
-  g_main_loop_quit (mainloop);
 
   g_main_context_pop_thread_default (self->thread_context);
 
@@ -966,6 +960,8 @@ static void
 notify_view_thread (GcalCalendarMonitor *self,
                     MonitorThreadEvent   event)
 {
+  g_assert (GCAL_IS_MAIN_THREAD ());
+
   g_async_queue_push (self->messages, GINT_TO_POINTER (event));
   g_main_context_wakeup (self->thread_context);
 }
@@ -975,6 +971,8 @@ maybe_spawn_view_thread (GcalCalendarMonitor *self)
 {
   g_autofree gchar *thread_name = NULL;
 
+  g_assert (GCAL_IS_MAIN_THREAD ());
+
   if (self->thread || !self->shared.range)
     return;
 
@@ -989,52 +987,59 @@ remove_events_outside_range (GcalCalendarMonitor *self,
                              GcalRange           *range)
 {
   g_autoptr (GRWLockWriterLocker) writer_locker = NULL;
+  g_autoptr (GPtrArray) events_to_remove = NULL;
   GHashTableIter iter;
   GcalEvent *event;
 
+  g_assert (GCAL_IS_MAIN_THREAD ());
+
   GCAL_TRACE_MSG ("Removing events outside range from monitor");
 
   writer_locker = g_rw_lock_writer_locker_new (&self->shared.lock);
 
+  events_to_remove = g_ptr_array_new_full (g_hash_table_size (self->shared.events),
+                                           g_object_unref);
+
   g_hash_table_iter_init (&iter, self->shared.events);
   while (g_hash_table_iter_next (&iter, NULL, (gpointer*) &event))
     {
-      g_object_ref (event);
+      GcalRange *event_range = gcal_event_get_range (event);
 
-      g_hash_table_iter_remove (&iter);
-      g_signal_emit (self, signals[EVENT_REMOVED], 0, event);
+      if (gcal_range_calculate_overlap (range, event_range, NULL) != GCAL_RANGE_NO_OVERLAP)
+        continue;
 
-      g_object_unref (event);
+      g_ptr_array_add (events_to_remove, event);
+      g_hash_table_iter_steal (&iter);
     }
+
+  if (events_to_remove->len > 0)
+    self->listener->remove_events (self, events_to_remove, self->listener_user_data);
 }
 
 static void
 remove_all_events (GcalCalendarMonitor *self)
 {
   g_autoptr (GRWLockWriterLocker) writer_locker = NULL;
-  GHashTableIter iter;
-  GcalEvent *event;
+  g_autoptr (GPtrArray) events_to_remove = NULL;
+
+  g_assert (GCAL_IS_MAIN_THREAD ());
 
   GCAL_TRACE_MSG ("Removing all events from view");
 
   writer_locker = g_rw_lock_writer_locker_new (&self->shared.lock);
 
-  g_hash_table_iter_init (&iter, self->shared.events);
-  while (g_hash_table_iter_next (&iter, NULL, (gpointer*) &event))
-    {
-      g_object_ref (event);
+  events_to_remove = g_hash_table_steal_all_values (self->shared.events);
 
-      g_hash_table_iter_remove (&iter);
-      g_signal_emit (self, signals[EVENT_REMOVED], 0, event);
-
-      g_object_unref (event);
-    }
+  if (events_to_remove->len > 0)
+    self->listener->remove_events (self, events_to_remove, self->listener_user_data);
 }
 
 static void
 set_complete (GcalCalendarMonitor *self,
               gboolean             complete)
 {
+  g_assert (GCAL_IS_MAIN_THREAD ());
+
   if (self->complete == complete)
     return;
 
@@ -1050,88 +1055,140 @@ set_complete (GcalCalendarMonitor *self,
  */
 
 static gboolean
-add_event_to_timeline_in_idle_cb (gpointer user_data)
+add_events_to_timeline_in_idle_cb (gpointer user_data)
 {
   g_autoptr (GRWLockWriterLocker) writer_locker = NULL;
+  g_autoptr (GPtrArray) events_to_add = NULL;
   GcalCalendarMonitor *self;
-  GcalEvent *event;
+  GPtrArray *events;
   IdleData *idle_data;
-  const gchar *event_id;
 
   GCAL_ENTRY;
 
+  g_assert (GCAL_IS_MAIN_THREAD ());
+
   idle_data = user_data;
   self = idle_data->monitor;
-  event = idle_data->event;
-  g_assert (idle_data->event_id == NULL);
+  events = idle_data->events;
+  g_assert (idle_data->event_ids == NULL);
 
-  event_id = gcal_event_get_uid (event);
+  events_to_add = g_ptr_array_sized_new (events->len);
 
   writer_locker = g_rw_lock_writer_locker_new (&self->shared.lock);
-  if (g_hash_table_insert (self->shared.events, g_strdup (event_id), g_object_ref (event)))
-    g_signal_emit (self, signals[EVENT_ADDED], 0, event);
+  for (guint i = 0; i < events->len; i++)
+    {
+      GcalEvent *event;
+      const gchar *uid;
+
+      event = g_ptr_array_index (events, i);
+      uid = gcal_event_get_uid (event);
+
+      if (!g_hash_table_contains (self->shared.events, uid))
+        {
+          g_hash_table_insert (self->shared.events, g_strdup (uid), g_object_ref (event));
+          g_ptr_array_add (events_to_add, event);
+        }
+    }
+
+  if (events_to_add->len > 0)
+    self->listener->add_events (self, events_to_add, self->listener_user_data);
 
   GCAL_RETURN (G_SOURCE_REMOVE);
 }
 
 static gboolean
-update_event_in_idle_cb (gpointer user_data)
+update_events_in_idle_cb (gpointer user_data)
 {
   g_autoptr (GRWLockWriterLocker) writer_locker = NULL;
+  g_autoptr (GPtrArray) old_events = NULL;
+  g_autoptr (GPtrArray) new_events = NULL;
+  g_autoptr (GPtrArray) events = NULL;
   GcalCalendarMonitor *self;
-  GcalEvent *old_event;
-  GcalEvent *event;
   IdleData *idle_data;
-  const gchar *event_id;
 
   GCAL_ENTRY;
 
+  g_assert (GCAL_IS_MAIN_THREAD ());
+
   idle_data = user_data;
   self = idle_data->monitor;
-  event = idle_data->event;
-  g_assert (idle_data->event_id == NULL);
+  events = g_steal_pointer (&idle_data->events);
+  g_assert (idle_data->event_ids == NULL);
 
-  event_id = gcal_event_get_uid (event);
+  old_events = g_ptr_array_new_full (events->len, g_object_unref);
+  new_events = g_ptr_array_sized_new (events->len);
 
   writer_locker = g_rw_lock_writer_locker_new (&self->shared.lock);
-  old_event = g_hash_table_lookup (self->shared.events, event_id);
-  if (old_event)
+  for (guint i = 0; i < events->len; i++)
     {
-      g_hash_table_insert (self->shared.events, g_strdup (event_id), g_object_ref (event));
-      g_signal_emit (self, signals[EVENT_UPDATED], 0, old_event, event);
+      g_autoptr (GcalEvent) old_event = NULL;
+      g_autofree gchar *old_event_id = NULL;
+      GcalEvent *event;
+      const gchar *event_id;
+
+      event = g_ptr_array_index (events, i);
+      event_id = gcal_event_get_uid (event);
+
+      if (g_hash_table_steal_extended (self->shared.events,
+                                       event_id,
+                                       (gpointer*) &old_event_id,
+                                       (gpointer*) &old_event))
+        {
+          g_hash_table_insert (self->shared.events, g_strdup (event_id), g_object_ref (event));
+          g_ptr_array_add (old_events, g_steal_pointer (&old_event));
+          g_ptr_array_add (new_events, event);
+        }
     }
 
+  g_assert (old_events->len == new_events->len);
+
+  if (new_events->len > 0)
+    self->listener->update_events (self, old_events, new_events, self->listener_user_data);
+
   GCAL_RETURN (G_SOURCE_REMOVE);
 }
 
 static gboolean
-remove_event_from_timeline_in_idle_cb (gpointer user_data)
+remove_events_from_timeline_in_idle_cb (gpointer user_data)
 {
   g_autoptr (GRWLockWriterLocker) writer_locker = NULL;
-  g_autoptr (GcalEvent) event = NULL;
+  g_autoptr (GPtrArray) events_to_remove = NULL;
+  g_autoptr (GPtrArray) event_ids = NULL;
   GcalCalendarMonitor *self;
   IdleData *idle_data;
-  gchar *event_id;
 
   GCAL_ENTRY;
 
+  g_assert (GCAL_IS_MAIN_THREAD ());
+
   idle_data = user_data;
   self = idle_data->monitor;
-  g_assert (idle_data->event == NULL);
-  event_id = idle_data->event_id;
+  event_ids = g_steal_pointer (&idle_data->event_ids);
+  g_assert (idle_data->events == NULL);
+
+  events_to_remove = g_ptr_array_new_full (event_ids->len, g_object_unref);
 
   writer_locker = g_rw_lock_writer_locker_new (&self->shared.lock);
-  event = g_hash_table_lookup (self->shared.events, event_id);
 
-  if (event)
+  for (guint i = 0; i < event_ids->len; i++)
     {
-      /* Keep the event alive until the signal is emitted */
-      g_object_ref (event);
+      const gchar *event_id;
+      GcalEvent *event;
 
-      g_hash_table_remove (self->shared.events, event_id);
-      g_signal_emit (self, signals[EVENT_REMOVED], 0, event);
+      event_id = g_ptr_array_index (event_ids, i);
+      event = g_hash_table_lookup (self->shared.events, event_id);
+
+      if (event)
+        {
+          /* Keep the event alive until the listener process it*/
+          g_ptr_array_add (events_to_remove, g_object_ref (event));
+          g_hash_table_remove (self->shared.events, event_id);
+        }
     }
 
+  if (events_to_remove->len > 0)
+    self->listener->remove_events (self, events_to_remove, self->listener_user_data);
+
   GCAL_RETURN (G_SOURCE_REMOVE);
 }
 
@@ -1159,10 +1216,12 @@ complete_in_idle_cb (gpointer user_data)
 
   GCAL_ENTRY;
 
+  g_assert (GCAL_IS_MAIN_THREAD ());
+
   idle_data = user_data;
   self = idle_data->monitor;
-  g_assert (idle_data->event == NULL);
-  g_assert (idle_data->event_id == NULL);
+  g_assert (idle_data->events == NULL);
+  g_assert (idle_data->event_ids == NULL);
 
   set_complete (self, idle_data->complete);
 
@@ -1175,16 +1234,31 @@ complete_in_idle_cb (gpointer user_data)
  */
 
 static void
-gcal_calendar_monitor_finalize (GObject *object)
+gcal_calendar_monitor_dispose (GObject *object)
 {
   GcalCalendarMonitor *self = (GcalCalendarMonitor *)object;
 
   g_cancellable_cancel (self->cancellable);
   notify_view_thread (self, QUIT);
-  g_clear_pointer (&self->thread, g_thread_join);
+
+  if (self->thread)
+    {
+      g_thread_join (self->thread);
+      self->thread = NULL;
+    }
 
   remove_all_events (self);
 
+  g_clear_object (&self->cancellable);
+
+  G_OBJECT_CLASS (gcal_calendar_monitor_parent_class)->dispose (object);
+}
+
+static void
+gcal_calendar_monitor_finalize (GObject *object)
+{
+  GcalCalendarMonitor *self = (GcalCalendarMonitor *)object;
+
   g_clear_object (&self->calendar);
   g_clear_pointer (&self->thread_context, g_main_context_unref);
   g_clear_pointer (&self->main_context, g_main_context_unref);
@@ -1249,35 +1323,11 @@ gcal_calendar_monitor_class_init (GcalCalendarMonitorClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->dispose = gcal_calendar_monitor_dispose;
   object_class->finalize = gcal_calendar_monitor_finalize;
   object_class->get_property = gcal_calendar_monitor_get_property;
   object_class->set_property = gcal_calendar_monitor_set_property;
 
-  signals[EVENT_ADDED] = g_signal_new ("event-added",
-                                       GCAL_TYPE_CALENDAR_MONITOR,
-                                       G_SIGNAL_RUN_FIRST,
-                                       0, NULL, NULL, NULL,
-                                       G_TYPE_NONE,
-                                       1,
-                                       GCAL_TYPE_EVENT);
-
-  signals[EVENT_UPDATED] = g_signal_new ("event-updated",
-                                         GCAL_TYPE_CALENDAR_MONITOR,
-                                         G_SIGNAL_RUN_FIRST,
-                                         0, NULL, NULL, NULL,
-                                         G_TYPE_NONE,
-                                         2,
-                                         GCAL_TYPE_EVENT,
-                                         GCAL_TYPE_EVENT);
-
-  signals[EVENT_REMOVED] = g_signal_new ("event-removed",
-                                         GCAL_TYPE_CALENDAR_MONITOR,
-                                         G_SIGNAL_RUN_FIRST,
-                                         0, NULL, NULL, NULL,
-                                         G_TYPE_NONE,
-                                         1,
-                                         GCAL_TYPE_EVENT);
-
   properties[PROP_CALENDAR] = g_param_spec_object ("calendar",
                                                    "Calendar",
                                                    "Calendar to be monitored",
@@ -1307,11 +1357,23 @@ gcal_calendar_monitor_init (GcalCalendarMonitor *self)
 }
 
 GcalCalendarMonitor*
-gcal_calendar_monitor_new (GcalCalendar *calendar)
+gcal_calendar_monitor_new (GcalCalendar                      *calendar,
+                           const GcalCalendarMonitorListener *listener,
+                           gpointer                           user_data)
 {
-  return g_object_new (GCAL_TYPE_CALENDAR_MONITOR,
-                       "calendar", calendar,
-                       NULL);
+  g_autoptr (GcalCalendarMonitor) monitor = NULL;
+
+  g_assert (calendar != NULL && GCAL_IS_CALENDAR (calendar));
+  g_assert (listener != NULL);
+
+  monitor = g_object_new (GCAL_TYPE_CALENDAR_MONITOR,
+                          "calendar", calendar,
+                          NULL);
+
+  monitor->listener = listener;
+  monitor->listener_user_data = user_data;
+
+  return g_steal_pointer (&monitor);
 }
 
 /**
