@@ -70,7 +70,7 @@ struct _GcalTimeline
   gchar              *filter;
 
   GHashTable         *calendars; /* GcalCalendar* -> GcalCalendarMonitor* */
-  guint               completed_calendars;
+  gboolean            complete;
 
   GHashTable         *subscribers; /* GcalTimelineSubscriber* -> SubscriberData* */
   GcalRangeTree      *subscriber_ranges;
@@ -188,10 +188,36 @@ queue_event_data (GcalTimeline           *self,
     }
 }
 
-static inline gboolean
-is_timeline_complete (GcalTimeline *self)
+static void
+update_completed_calendars (GcalTimeline *self)
 {
-  return self->completed_calendars == g_hash_table_size (self->calendars);
+  GcalCalendarMonitor *calendar_monitor;
+  GcalCalendar *calendar;
+  GHashTableIter iter;
+  gboolean was_complete;
+  gboolean is_complete;
+
+  GCAL_ENTRY;
+
+  was_complete = self->complete;
+  is_complete = TRUE;
+
+  g_hash_table_iter_init (&iter, self->calendars);
+  while (g_hash_table_iter_next (&iter, (gpointer*) &calendar, (gpointer*) &calendar_monitor))
+    {
+      if (gcal_calendar_get_visible (calendar))
+        is_complete &= gcal_calendar_monitor_is_complete (calendar_monitor);
+    }
+
+  if (was_complete != is_complete)
+    {
+      GCAL_TRACE_MSG ("Timeline %p is %s", self, is_complete ? "complete" : "incomplete");
+
+      self->complete = is_complete;
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_COMPLETE]);
+    }
+
+  GCAL_EXIT;
 }
 
 static void
@@ -207,6 +233,7 @@ add_event_to_subscriber (GcalTimelineSubscriber *subscriber,
 
 static void
 update_subscriber_event (GcalTimelineSubscriber *subscriber,
+                         GcalEvent              *old_event,
                          GcalEvent              *event)
 {
   GCAL_TRACE_MSG ("Updating event '%s' (%s) at subscriber %s",
@@ -214,7 +241,7 @@ update_subscriber_event (GcalTimelineSubscriber *subscriber,
                   gcal_event_get_uid (event),
                   G_OBJECT_TYPE_NAME (subscriber));
 
-  GCAL_TIMELINE_SUBSCRIBER_GET_IFACE (subscriber)->update_event (subscriber, event);
+  GCAL_TIMELINE_SUBSCRIBER_GET_IFACE (subscriber)->update_event (subscriber, old_event, event);
 }
 static void
 remove_event_from_subscriber (GcalTimelineSubscriber *subscriber,
@@ -473,97 +500,135 @@ update_calendar_monitor_filters (GcalTimeline *self)
     gcal_calendar_monitor_set_filter (monitor, self->filter);
 }
 
+/*
+ * GcalCalendarMonitorListener events
+ */
+
+static void
+calendar_monitor_add_events_cb (GcalCalendarMonitor *monitor,
+                                GPtrArray           *events,
+                                gpointer             user_data)
+{
+  GcalTimeline *self;
+
+  GCAL_ENTRY;
+
+  self = GCAL_TIMELINE (user_data);
+
+  for (guint i = 0; i < events->len; i++)
+    {
+      g_autoptr (GPtrArray) subscribers_at_range = NULL;
+      GcalRange *event_range;
+      GcalEvent *event;
+
+      event = g_ptr_array_index (events, i);
+      event_range = gcal_event_get_range (event);
+
+      /* Add to all subscribers within the event range */
+      subscribers_at_range = gcal_range_tree_get_data_at_range (self->subscriber_ranges, event_range);
+
+      queue_event_data (self, ADD_EVENT, NULL, event, NULL, TRUE);
+
+      for (guint j = 0; subscribers_at_range && j < subscribers_at_range->len; j++)
+        {
+          GcalTimelineSubscriber *subscriber = g_ptr_array_index (subscribers_at_range, j);
+
+          queue_event_data (self, ADD_EVENT, subscriber, event, NULL, FALSE);
+        }
+    }
+
+  GCAL_EXIT;
+}
+
+static void
+calendar_monitor_update_events_cb (GcalCalendarMonitor *monitor,
+                                   GPtrArray           *old_events,
+                                   GPtrArray           *events,
+                                   gpointer             user_data)
+{
+  GcalTimeline *self;
+
+  GCAL_ENTRY;
+
+  g_assert (old_events->len == events->len);
+
+  self = GCAL_TIMELINE (user_data);
+
+  for (guint i = 0; i < events->len; i++)
+    {
+      g_autoptr (GPtrArray) subscribers_at_range = NULL;
+      GcalRange *event_range;
+      GcalEvent *old_event;
+      GcalEvent *event;
+
+      event = g_ptr_array_index (events, i);
+      old_event = g_ptr_array_index (old_events, i);
+      event_range = gcal_event_get_range (event);
+
+      /* Add to all subscribers within the event range */
+      subscribers_at_range = gcal_range_tree_get_data_at_range (self->subscriber_ranges, event_range);
+
+      queue_event_data (self, UPDATE_EVENT, NULL, event, old_event, TRUE);
+
+      for (guint j = 0; subscribers_at_range && j < subscribers_at_range->len; j++)
+        {
+          GcalTimelineSubscriber *subscriber = g_ptr_array_index (subscribers_at_range, j);
+
+          queue_event_data (self, UPDATE_EVENT, subscriber, event, old_event, FALSE);
+        }
+
+    }
+
+  GCAL_EXIT;
+}
+
+static void
+calendar_monitor_remove_events_cb (GcalCalendarMonitor *monitor,
+                                   GPtrArray           *events,
+                                   gpointer             user_data)
+{
+  GcalTimeline *self;
+
+  GCAL_ENTRY;
+
+  self = GCAL_TIMELINE (user_data);
+
+  for (guint i = 0; i < events->len; i++)
+    {
+      g_autoptr (GPtrArray) subscribers_at_range = NULL;
+      GcalRange *event_range;
+      GcalEvent *event;
+
+      event = g_ptr_array_index (events, i);
+      event_range = gcal_event_get_range (event);
+
+      /* Add to all subscribers within the event range */
+      subscribers_at_range = gcal_range_tree_get_data_at_range (self->subscriber_ranges, event_range);
+
+      for (guint j = 0; subscribers_at_range && j < subscribers_at_range->len; j++)
+        {
+          GcalTimelineSubscriber *subscriber = g_ptr_array_index (subscribers_at_range, j);
+
+          queue_event_data (self, REMOVE_EVENT, subscriber, event, NULL, FALSE);
+        }
+
+      queue_event_data (self, REMOVE_EVENT, NULL, event, NULL, TRUE);
+    }
+
+  GCAL_EXIT;
+}
+
+static const GcalCalendarMonitorListener monitor_listener = {
+  calendar_monitor_add_events_cb,
+  calendar_monitor_update_events_cb,
+  calendar_monitor_remove_events_cb,
+};
+
 
 /*
  * Callbacks
  */
 
-static void
-on_calendar_monitor_event_added_cb (GcalCalendarMonitor *monitor,
-                                    GcalEvent           *event,
-                                    GcalTimeline        *self)
-{
-  g_autoptr (GPtrArray) subscribers_at_range = NULL;
-  GcalRange *event_range;
-  gint i;
-
-  GCAL_ENTRY;
-
-  event_range = gcal_event_get_range (event);
-
-  /* Add to all subscribers within the event range */
-  subscribers_at_range = gcal_range_tree_get_data_at_range (self->subscriber_ranges, event_range);
-
-  queue_event_data (self, ADD_EVENT, NULL, event, NULL, TRUE);
-
-  for (i = 0; subscribers_at_range && i < subscribers_at_range->len; i++)
-    {
-      GcalTimelineSubscriber *subscriber = g_ptr_array_index (subscribers_at_range, i);
-
-      queue_event_data (self, ADD_EVENT, subscriber, event, NULL, FALSE);
-    }
-
-  GCAL_EXIT;
-}
-
-static void
-on_calendar_monitor_event_updated_cb (GcalCalendarMonitor *monitor,
-                                      GcalEvent           *old_event,
-                                      GcalEvent           *event,
-                                      GcalTimeline        *self)
-{
-  g_autoptr (GPtrArray) subscribers_at_range = NULL;
-  GcalRange *event_range;
-  gint i;
-
-  GCAL_ENTRY;
-
-  /* Add the updated event to the (potentially new) range */
-  event_range = gcal_event_get_range (event);
-
-  /* Add to all subscribers within the event range */
-  subscribers_at_range = gcal_range_tree_get_data_at_range (self->subscriber_ranges, event_range);
-
-  queue_event_data (self, UPDATE_EVENT, NULL, event, old_event, TRUE);
-
-  for (i = 0; subscribers_at_range && i < subscribers_at_range->len; i++)
-    {
-      GcalTimelineSubscriber *subscriber = g_ptr_array_index (subscribers_at_range, i);
-
-      queue_event_data (self, UPDATE_EVENT, subscriber, event, old_event, FALSE);
-    }
-
-  GCAL_EXIT;
-}
-
-static void
-on_calendar_monitor_event_removed_cb (GcalCalendarMonitor *monitor,
-                                      GcalEvent           *event,
-                                      GcalTimeline        *self)
-{
-  g_autoptr (GPtrArray) subscribers_at_range = NULL;
-  GcalRange *event_range;
-  gint i;
-
-  GCAL_ENTRY;
-
-  event_range = gcal_event_get_range (event);
-
-  /* Add to all subscribers within the event range */
-  subscribers_at_range = gcal_range_tree_get_data_at_range (self->subscriber_ranges, event_range);
-
-  for (i = 0; subscribers_at_range && i < subscribers_at_range->len; i++)
-    {
-      GcalTimelineSubscriber *subscriber = g_ptr_array_index (subscribers_at_range, i);
-
-      queue_event_data (self, REMOVE_EVENT, subscriber, event, NULL, FALSE);
-    }
-
-  queue_event_data (self, REMOVE_EVENT, NULL, event, NULL, TRUE);
-
-  GCAL_EXIT;
-}
-
 static void
 on_calendar_monitor_completed_cb (GcalCalendarMonitor *monitor,
                                   GParamSpec          *pspec,
@@ -571,15 +636,7 @@ on_calendar_monitor_completed_cb (GcalCalendarMonitor *monitor,
 {
   GCAL_ENTRY;
 
-  if (gcal_calendar_monitor_is_complete (monitor))
-    self->completed_calendars++;
-  else
-    self->completed_calendars--;
-
-  g_assert (self->completed_calendars <= g_hash_table_size (self->calendars));
-
-  if (is_timeline_complete (self))
-    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_COMPLETE]);
+  update_completed_calendars (self);
 
   GCAL_EXIT;
 }
@@ -703,7 +760,7 @@ timeline_source_dispatch (GSource     *source,
               }
 
             if (subscriber)
-              update_subscriber_event (subscriber, event);
+              update_subscriber_event (subscriber, queue_data->old_event, event);
           }
           break;
 
@@ -784,7 +841,7 @@ gcal_timeline_get_property (GObject    *object,
   switch (prop_id)
     {
     case PROP_COMPLETE:
-      g_value_set_boolean (value, is_timeline_complete (self));
+      g_value_set_boolean (value, self->complete);
       break;
 
     case PROP_CONTEXT:
@@ -933,16 +990,15 @@ gcal_timeline_add_calendar (GcalTimeline *self,
 
   GCAL_TRACE_MSG ("Adding calendar '%s' to timeline %p", gcal_calendar_get_name (calendar), self);
 
-  monitor = gcal_calendar_monitor_new (calendar);
-  g_signal_connect (monitor, "event-added", G_CALLBACK (on_calendar_monitor_event_added_cb), self);
-  g_signal_connect (monitor, "event-updated", G_CALLBACK (on_calendar_monitor_event_updated_cb), self);
-  g_signal_connect (monitor, "event-removed", G_CALLBACK (on_calendar_monitor_event_removed_cb), self);
+  monitor = gcal_calendar_monitor_new (calendar, &monitor_listener, self);
   g_signal_connect (monitor, "notify::complete", G_CALLBACK (on_calendar_monitor_completed_cb), self);
   g_hash_table_insert (self->calendars, calendar, g_object_ref (monitor));
 
   if (self->range)
     gcal_calendar_monitor_set_range (monitor, self->range);
 
+  update_completed_calendars (self);
+
   GCAL_EXIT;
 }
 
@@ -958,7 +1014,10 @@ gcal_timeline_remove_calendar (GcalTimeline *self,
   g_object_ref (calendar);
 
   if (g_hash_table_remove (self->calendars, calendar))
-    GCAL_TRACE_MSG ("Removing calendar '%s' from timeline %p", gcal_calendar_get_name (calendar), self);
+    {
+      GCAL_TRACE_MSG ("Removing calendar '%s' from timeline %p", gcal_calendar_get_name (calendar), self);
+      update_completed_calendars (self);
+    }
 
   g_object_unref (calendar);
 
@@ -1013,9 +1072,6 @@ gcal_timeline_remove_subscriber (GcalTimeline           *self,
   g_signal_handlers_disconnect_by_func (subscriber, on_subscriber_range_changed_cb, self);
   g_hash_table_remove (self->subscribers, subscriber);
 
-  if (g_hash_table_size (self->subscribers) == 0)
-    g_queue_clear_full (self->event_queue, (GDestroyNotify) queue_data_free);
-
   gcal_range_tree_remove_data (self->subscriber_ranges, subscriber);
   update_range (self);
 
@@ -1074,5 +1130,5 @@ gcal_timeline_is_complete (GcalTimeline *self)
 {
   g_return_val_if_fail (GCAL_IS_TIMELINE (self), FALSE);
 
-  return is_timeline_complete (self);
+  return self->complete;
 }
