@@ -30,16 +30,14 @@
 
 struct _GcalCalendarsPage
 {
-  GtkBox              parent;
+  AdwBin              parent;
 
   GtkListBoxRow      *add_calendar_row;
   GtkListBox         *listbox;
-  GtkLabel           *notification_label;
-  GtkRevealer        *notification_revealer;
   GtkSizeGroup       *sizegroup;
+  AdwToastOverlay    *toast_overlay;
 
-  GcalCalendar       *removed_calendar;
-  gint                notification_timeout_id;
+  AdwToast           *toast;
 
   GcalContext        *context;
 };
@@ -50,7 +48,7 @@ static void          on_calendar_color_changed_cb                (GcalCalendar
                                                                   GParamSpec         *pspec,
                                                                   GtkImage           *icon);
 
-G_DEFINE_TYPE_WITH_CODE (GcalCalendarsPage, gcal_calendars_page, GTK_TYPE_BOX,
+G_DEFINE_TYPE_WITH_CODE (GcalCalendarsPage, gcal_calendars_page, ADW_TYPE_BIN,
                          G_IMPLEMENT_INTERFACE (GCAL_TYPE_CALENDAR_MANAGEMENT_PAGE,
                                                 gcal_calendar_management_page_iface_init))
 
@@ -70,13 +68,11 @@ static GtkWidget*
 make_calendar_row (GcalCalendarsPage *self,
                    GcalCalendar      *calendar)
 {
+  g_autoptr (GdkPaintable) color_paintable = NULL;
   g_autofree gchar *parent_name = NULL;
   g_autoptr (GtkBuilder) builder = NULL;
-  cairo_surface_t *surface;
   const GdkRGBA *color;
   GcalManager *manager;
-  GtkWidget *bottom_label;
-  GtkWidget *top_label;
   GtkWidget *icon;
   GtkWidget *row;
   GtkWidget *sw;
@@ -95,14 +91,14 @@ make_calendar_row (GcalCalendarsPage *self,
 
   /* source color icon */
   color = gcal_calendar_get_color (calendar);
-  surface = get_circle_surface_from_color (color, 24);
+  color_paintable = get_circle_paintable_from_color (color, 24);
   icon = GTK_WIDGET (gtk_builder_get_object (builder, "icon"));
-  gtk_image_set_from_surface (GTK_IMAGE (icon), surface);
+  gtk_image_set_from_paintable (GTK_IMAGE (icon), color_paintable);
 
   /* source name label */
-  top_label = GTK_WIDGET (gtk_builder_get_object (builder, "title"));
-  gtk_label_set_label (GTK_LABEL (top_label), gcal_calendar_get_name (calendar));
-  g_object_bind_property (calendar, "name", top_label, "label", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+  adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), gcal_calendar_get_name (calendar));
+  g_object_bind_property (calendar, "name", row, "title", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+
   g_signal_connect_object (calendar,
                            "notify::name",
                            G_CALLBACK (gtk_list_box_invalidate_sort),
@@ -120,10 +116,7 @@ make_calendar_row (GcalCalendarsPage *self,
   g_object_bind_property (calendar, "visible", sw, "active", G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
   /* parent source name label */
-  bottom_label = GTK_WIDGET (gtk_builder_get_object (builder, "subtitle"));
-  gtk_label_set_label (GTK_LABEL (bottom_label), parent_name);
-
-  g_clear_pointer (&surface, cairo_surface_destroy);
+  adw_action_row_set_subtitle (ADW_ACTION_ROW (row), parent_name);
 
   gtk_size_group_add_widget (self->sizegroup, row);
 
@@ -134,16 +127,15 @@ static void
 add_calendar (GcalCalendarsPage *self,
               GcalCalendar      *calendar)
 {
-  g_autoptr (GList) children = NULL;
+  GtkWidget *child;
   GtkWidget *row;
   ESource *source;
-  GList *l;
 
-  children = gtk_container_get_children (GTK_CONTAINER (self->listbox));
-
-  for (l = children; l; l = l->next)
+  for (child = gtk_widget_get_first_child (GTK_WIDGET (self->listbox));
+       child != NULL;
+       child = gtk_widget_get_next_sibling (child))
     {
-      if (g_object_get_data (l->data, "calendar") == calendar)
+      if (g_object_get_data (G_OBJECT (child), "calendar") == calendar)
         return;
     }
 
@@ -152,7 +144,7 @@ add_calendar (GcalCalendarsPage *self,
   row = make_calendar_row (self, calendar);
   g_object_set_data (G_OBJECT (row), "source", source);
   g_object_set_data (G_OBJECT (row), "calendar", calendar);
-  gtk_container_add (GTK_CONTAINER (self->listbox), row);
+  gtk_list_box_append (self->listbox, row);
 }
 
 
@@ -160,18 +152,17 @@ static void
 remove_calendar (GcalCalendarsPage *self,
                  GcalCalendar      *calendar)
 {
-  g_autoptr (GList) children = NULL;
-  GList *aux;
+  GtkWidget *child;
 
-  children = gtk_container_get_children (GTK_CONTAINER (self->listbox));
-
-  for (aux = children; aux != NULL; aux = aux->next)
+  for (child = gtk_widget_get_first_child (GTK_WIDGET (self->listbox));
+       child != NULL;
+       child = gtk_widget_get_next_sibling (child))
     {
-      GcalCalendar *row_calendar = g_object_get_data (G_OBJECT (aux->data), "calendar");
+      GcalCalendar *row_calendar = g_object_get_data (G_OBJECT (child), "calendar");
 
       if (row_calendar && row_calendar == calendar)
         {
-          gtk_widget_destroy (aux->data);
+          gtk_list_box_remove (self->listbox, child);
           break;
         }
     }
@@ -193,17 +184,14 @@ delete_calendar (GcalCalendarsPage *self,
     return;
 
   /* Enable the source again to remove it's name from disabled list */
-  gcal_calendar_set_visible (self->removed_calendar, TRUE);
+  gcal_calendar_set_visible (calendar, TRUE);
   e_source_remove_sync (removed_source, NULL, &error);
 
   if (error != NULL)
     {
       g_warning ("[source-dialog] Error removing source: %s", error->message);
-      add_calendar (self, self->removed_calendar);
+      add_calendar (self, calendar);
     }
-
-  gtk_revealer_set_reveal_child (self->notification_revealer, FALSE);
-  g_clear_handle_id (&self->notification_timeout_id, g_source_remove);
 }
 
 
@@ -248,47 +236,17 @@ listbox_sort_func (GtkListBoxRow *row1,
   return g_strcmp0 (parent_name1, parent_name2);
 }
 
-static gboolean
-remove_calendar_after_delay_cb (gpointer data)
-{
-  GcalCalendarsPage *self = GCAL_CALENDARS_PAGE (data);
-
-  g_assert (self->removed_calendar != NULL);
-
-  delete_calendar (self, self->removed_calendar);
-  g_clear_object (&self->removed_calendar);
-
-  gtk_revealer_set_reveal_child (self->notification_revealer, FALSE);
-  self->notification_timeout_id = 0;
-
-  return G_SOURCE_REMOVE;
-}
-
 static void
 on_calendar_color_changed_cb (GcalCalendar *calendar,
                               GParamSpec   *pspec,
                               GtkImage     *icon)
 {
-  cairo_surface_t *surface;
+  g_autoptr (GdkPaintable) color_paintable = NULL;
   const GdkRGBA *color;
 
   color = gcal_calendar_get_color (calendar);
-  surface = get_circle_surface_from_color (color, 24);
-  gtk_image_set_from_surface (GTK_IMAGE (icon), surface);
-  g_clear_pointer (&surface, cairo_surface_destroy);
-}
-
-static void
-on_close_notification_button_clicked_cb (GtkWidget         *button,
-                                         GcalCalendarsPage *self)
-{
-  g_assert (self->removed_calendar != NULL);
-
-  delete_calendar (self, self->removed_calendar);
-  g_clear_object (&self->removed_calendar);
-
-  gtk_revealer_set_reveal_child (self->notification_revealer, FALSE);
-  g_clear_handle_id (&self->notification_timeout_id, g_source_remove);
+  color_paintable = get_circle_paintable_from_color (color, 24);
+  gtk_image_set_from_paintable (GTK_IMAGE (icon), color_paintable);
 }
 
 static void
@@ -327,19 +285,47 @@ on_manager_calendar_removed_cb (GcalManager       *manager,
 }
 
 static void
-on_undo_remove_button_clicked_cb (GtkButton         *button,
-                                  GcalCalendarsPage *self)
+on_toast_button_clicked_cb (AdwToast          *toast,
+                            GcalCalendarsPage *self)
 {
-  if (!self->removed_calendar)
-    return;
+  GcalCalendar *calendar;
 
-  gcal_calendar_set_visible (self->removed_calendar, TRUE);
+  GCAL_ENTRY;
 
-  add_calendar (self, self->removed_calendar);
-  g_clear_object (&self->removed_calendar);
+  calendar = g_object_get_data (G_OBJECT (toast), "calendar");
+  g_assert (calendar != NULL);
 
-  gtk_revealer_set_reveal_child (self->notification_revealer, FALSE);
-  g_clear_handle_id (&self->notification_timeout_id, g_source_remove);
+  gcal_calendar_set_visible (calendar, TRUE);
+  add_calendar (self, calendar);
+
+  g_object_set_data (G_OBJECT (toast), "calendar", NULL);
+
+  g_clear_object (&self->toast);
+
+  GCAL_EXIT;
+}
+
+static void
+on_toast_dismissed_cb (AdwToast          *toast,
+                       GcalCalendarsPage *self)
+{
+  GcalCalendar *calendar;
+
+  GCAL_ENTRY;
+
+  calendar = g_object_get_data (G_OBJECT (toast), "calendar");
+
+  if (!calendar)
+    {
+      g_assert (self->toast == NULL);
+      GCAL_RETURN ();
+    }
+
+  delete_calendar (self, calendar);
+
+  g_clear_object (&self->toast);
+
+  GCAL_EXIT;
 }
 
 
@@ -356,7 +342,7 @@ gcal_calendars_page_get_name (GcalCalendarManagementPage *page)
 static const gchar*
 gcal_calendars_page_get_title (GcalCalendarManagementPage *page)
 {
-  return _("Manage Calendars");
+  return _("Calendars");
 }
 
 static void
@@ -364,6 +350,7 @@ gcal_calendars_page_activate (GcalCalendarManagementPage *page,
                               GcalCalendar               *calendar)
 {
   g_autofree gchar *new_string = NULL;
+  g_autoptr (AdwToast) toast = NULL;
   GcalCalendarsPage *self;
 
   GCAL_ENTRY;
@@ -374,32 +361,26 @@ gcal_calendars_page_activate (GcalCalendarManagementPage *page,
   self = GCAL_CALENDARS_PAGE (page);
 
   /* Remove the previously deleted calendar, if any */
-  if (self->removed_calendar)
-    {
-      delete_calendar (self, self->removed_calendar);
-      g_clear_object (&self->removed_calendar);
-    }
-
-  self->removed_calendar = g_object_ref (calendar);
+  g_clear_pointer (&self->toast, adw_toast_dismiss);
 
   /* Remove the listbox entry (if any) */
   remove_calendar (self, calendar);
 
-  /* Update notification label */
+  /* Create the new toast */
   new_string = g_markup_printf_escaped (_("Calendar <b>%s</b> removed"),
-                                        gcal_calendar_get_name (self->removed_calendar));
-  gtk_label_set_markup (self->notification_label, new_string);
+                                        gcal_calendar_get_name (calendar));
 
-  gtk_revealer_set_reveal_child (self->notification_revealer, TRUE);
+  toast = adw_toast_new (new_string);
+  adw_toast_set_timeout (toast, 7);
+  adw_toast_set_button_label (toast, _("_Undo"));
+  g_object_set_data_full (G_OBJECT (toast), "calendar", g_object_ref (calendar), g_object_unref);
+  g_signal_connect (toast, "dismissed", G_CALLBACK (on_toast_dismissed_cb), self);
+  g_signal_connect (toast, "button-clicked", G_CALLBACK (on_toast_button_clicked_cb), self);
+  adw_toast_overlay_add_toast (self->toast_overlay, g_object_ref (toast));
 
-  /* Remove old notifications */
-  if (self->notification_timeout_id != 0)
-    g_source_remove (self->notification_timeout_id);
-
-  self->notification_timeout_id = g_timeout_add_seconds (7, remove_calendar_after_delay_cb, self);
-
-  gcal_calendar_set_visible (self->removed_calendar, FALSE);
+  self->toast = g_steal_pointer (&toast);
 
+  gcal_calendar_set_visible (calendar, FALSE);
 
   GCAL_EXIT;
 }
@@ -421,7 +402,7 @@ gcal_calendars_page_finalize (GObject *object)
 {
   GcalCalendarsPage *self = (GcalCalendarsPage *)object;
 
-  g_clear_handle_id (&self->notification_timeout_id, g_source_remove);
+  g_clear_object (&self->toast);
   g_clear_object (&self->context);
 
   G_OBJECT_CLASS (gcal_calendars_page_parent_class)->finalize (object);
@@ -496,13 +477,10 @@ gcal_calendars_page_class_init (GcalCalendarsPageClass *klass)
 
   gtk_widget_class_bind_template_child (widget_class, GcalCalendarsPage, add_calendar_row);
   gtk_widget_class_bind_template_child (widget_class, GcalCalendarsPage, listbox);
-  gtk_widget_class_bind_template_child (widget_class, GcalCalendarsPage, notification_label);
-  gtk_widget_class_bind_template_child (widget_class, GcalCalendarsPage, notification_revealer);
   gtk_widget_class_bind_template_child (widget_class, GcalCalendarsPage, sizegroup);
+  gtk_widget_class_bind_template_child (widget_class, GcalCalendarsPage, toast_overlay);
 
-  gtk_widget_class_bind_template_callback (widget_class, on_close_notification_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_listbox_row_activated_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_undo_remove_button_clicked_cb);
 }
 
 static void
