@@ -24,8 +24,9 @@
 #include "gcal-clock.h"
 #include "gcal-debug.h"
 #include "gcal-event-widget.h"
-#include "gcal-utils.h"
 #include "gcal-month-cell.h"
+#include "gcal-utils.h"
+#include "gcal-weather-info.h"
 
 struct _GcalMonthCell
 {
@@ -36,11 +37,12 @@ struct _GcalMonthCell
 
   GtkLabel           *day_label;
   GtkWidget          *header_box;
+  GtkLabel           *month_name_label;
   GtkImage           *weather_icon;
   GtkLabel           *temp_label;
 
   GtkWidget          *overflow_button;
-  GtkWidget          *overflow_label;
+  GtkInscription     *overflow_inscription;
   GtkWidget          *overlay;
 
   gboolean            different_month;
@@ -78,6 +80,7 @@ static void
 update_style_flags (GcalMonthCell *self)
 {
   g_autoptr (GDateTime) today = NULL;
+  gint weekday;
 
   /* Today */
   today = g_date_time_new_now_local ();
@@ -86,6 +89,12 @@ update_style_flags (GcalMonthCell *self)
     gtk_widget_add_css_class (GTK_WIDGET (self), "today");
   else
     gtk_widget_remove_css_class (GTK_WIDGET (self), "today");
+
+  weekday = g_date_time_get_day_of_week (self->date);
+  if (is_workday (weekday))
+    gtk_widget_add_css_class (GTK_WIDGET (self), "workday");
+  else
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "workday");
 }
 
 static void
@@ -123,9 +132,9 @@ move_event (GcalMonthCell         *self,
                                    current_month - start_month,
                                    0, 0, 0, 0);
 
-  diff = g_date_time_get_day_of_month (self->date) - g_date_time_get_day_of_month (start_dt);
+  diff = gcal_date_time_compare_date (self->date, start_dt);
 
-  if (diff != 0 || current_month != start_month || current_year != start_year)
+  if (diff != 0)
     {
       g_autoptr (GDateTime) new_start = g_date_time_add_days (start_dt, diff);
 
@@ -143,6 +152,49 @@ move_event (GcalMonthCell         *self,
     }
 }
 
+static void
+update_weather (GcalMonthCell *self)
+{
+  GcalWeatherService *weather_service;
+  GcalWeatherInfo *weather_info;
+  GDate date;
+
+  if (!self->context)
+    return;
+
+  g_date_set_dmy (&date,
+                  g_date_time_get_day_of_month (self->date),
+                  g_date_time_get_month (self->date),
+                  g_date_time_get_year (self->date));
+
+  weather_service = gcal_context_get_weather_service (self->context);
+  weather_info = gcal_weather_service_get_weather_info_for_date (weather_service, &date);
+
+  g_assert (!weather_info || GCAL_IS_WEATHER_INFO (weather_info));
+
+  if (self->weather_info == weather_info)
+    return;
+
+  self->weather_info = weather_info;
+
+  if (weather_info)
+    {
+      const gchar *icon_name; /* unowned */
+      const gchar *temp_str;  /* unwoned */
+
+      icon_name = gcal_weather_info_get_icon_name (weather_info);
+      temp_str = gcal_weather_info_get_temperature (weather_info);
+
+      gtk_image_set_from_icon_name (self->weather_icon, icon_name);
+      gtk_label_set_text (self->temp_label, temp_str);
+    }
+  else
+    {
+      gtk_image_clear (self->weather_icon);
+      gtk_label_set_text (self->temp_label, "");
+    }
+}
+
 
 /*
  * Callbacks
@@ -175,7 +227,7 @@ on_drop_target_accept_cb (GtkDropTarget *drop_target,
   if (!gdk_content_formats_contain_gtype (gdk_drop_get_formats (drop), GCAL_TYPE_EVENT_WIDGET))
     GCAL_RETURN (FALSE);
 
-  GCAL_RETURN (!self->different_month);
+  GCAL_RETURN (TRUE);
 }
 
 static void
@@ -201,9 +253,6 @@ on_drop_target_drop_cb (GtkDropTarget *drop_target,
 
   GCAL_ENTRY;
 
-  if (self->different_month)
-    GCAL_RETURN (FALSE);
-
   if (!G_VALUE_HOLDS (value, GCAL_TYPE_EVENT_WIDGET))
     GCAL_RETURN (FALSE);
 
@@ -227,6 +276,13 @@ on_drop_target_drop_cb (GtkDropTarget *drop_target,
   GCAL_RETURN (TRUE);
 }
 
+static void
+on_weather_service_weather_changed_cb (GcalWeatherService *weather_service,
+                                       GcalMonthCell      *self)
+{
+  update_weather (self);
+}
+
 
 /*
  * GObject overrides
@@ -314,8 +370,9 @@ gcal_month_cell_class_init (GcalMonthCellClass *klass)
 
   gtk_widget_class_bind_template_child (widget_class, GcalMonthCell, day_label);
   gtk_widget_class_bind_template_child (widget_class, GcalMonthCell, header_box);
+  gtk_widget_class_bind_template_child (widget_class, GcalMonthCell, month_name_label);
   gtk_widget_class_bind_template_child (widget_class, GcalMonthCell, overflow_button);
-  gtk_widget_class_bind_template_child (widget_class, GcalMonthCell, overflow_label);
+  gtk_widget_class_bind_template_child (widget_class, GcalMonthCell, overflow_inscription);
   gtk_widget_class_bind_template_child (widget_class, GcalMonthCell, overlay);
   gtk_widget_class_bind_template_child (widget_class, GcalMonthCell, temp_label);
   gtk_widget_class_bind_template_child (widget_class, GcalMonthCell, weather_icon);
@@ -358,6 +415,7 @@ gcal_month_cell_set_date (GcalMonthCell *self,
                           GDateTime     *date)
 {
   g_autofree gchar *text = NULL;
+  gint day_of_month;
 
   g_return_if_fail (GCAL_IS_MONTH_CELL (self));
 
@@ -372,46 +430,17 @@ gcal_month_cell_set_date (GcalMonthCell *self,
 
   gtk_label_set_text (self->day_label, text);
   update_style_flags (self);
-}
 
-/**
- * gcal_month_cell_set_weather:
- * @self: The #GcalMonthCell instance.
- * @info: (nullable): The weather information to display.
- *
- * Sets the weather information to display for this day.
- *
- * Note that this function does not check dates nor
- * manages weather information changes on its own.
- */
-void
-gcal_month_cell_set_weather (GcalMonthCell   *self,
-                             GcalWeatherInfo *info)
-{
-  g_return_if_fail (GCAL_IS_MONTH_CELL (self));
-  g_return_if_fail (!info || GCAL_IS_WEATHER_INFO (info));
-
-  if (self->weather_info == info)
-    return;
-
-  self->weather_info = info;
-
-  if (!info)
+  /* Month name */
+  day_of_month = g_date_time_get_day_of_month (date);
+  gtk_widget_set_visible (GTK_WIDGET (self->month_name_label), day_of_month == 1);
+  if (g_date_time_get_day_of_month (date) == 1)
     {
-      gtk_image_clear (self->weather_icon);
-      gtk_label_set_text (self->temp_label, "");
+      g_autofree gchar *month_name = g_date_time_format (date, "%b");
+      gtk_label_set_text (self->month_name_label, month_name);
     }
-  else
-    {
-      const gchar* icon_name; /* unowned */
-      const gchar* temp_str;  /* unwoned */
 
-      icon_name = gcal_weather_info_get_icon_name (info);
-      temp_str = gcal_weather_info_get_temperature (info);
-
-      gtk_image_set_from_icon_name (self->weather_icon, icon_name);
-      gtk_label_set_text (self->temp_label, temp_str);
-    }
+  update_weather (self);
 }
 
 gboolean
@@ -462,6 +491,14 @@ gcal_month_cell_set_context (GcalMonthCell *self,
                            self,
                            0);
 
+  g_signal_connect_object (gcal_context_get_weather_service (self->context),
+                           "weather-changed",
+                           G_CALLBACK (on_weather_service_weather_changed_cb),
+                           self,
+                           0);
+
+  update_weather (self);
+
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_CONTEXT]);
 }
 
@@ -477,20 +514,24 @@ void
 gcal_month_cell_set_overflow (GcalMonthCell *self,
                               guint          n_overflow)
 {
-  g_autofree gchar *text = NULL;
-
   g_return_if_fail (GCAL_IS_MONTH_CELL (self));
 
-  gtk_widget_set_visible (self->overflow_label, n_overflow > 0);
-  gtk_widget_set_sensitive (self->overflow_button, n_overflow > 0);
-
   if (self->n_overflow == n_overflow)
     return;
 
   self->n_overflow = n_overflow;
 
-  text = g_strdup_printf ("+%d", n_overflow);
-  gtk_label_set_text (GTK_LABEL (self->overflow_label), text);
+  gtk_widget_set_sensitive (self->overflow_button, n_overflow > 0);
+
+  if (n_overflow > 0)
+    {
+      g_autofree gchar *text = g_strdup_printf ("+%d", n_overflow);
+      gtk_inscription_set_text (self->overflow_inscription, text);
+    }
+  else
+    {
+      gtk_inscription_set_text (self->overflow_inscription, "");
+    }
 }
 
 gint
@@ -506,7 +547,7 @@ gcal_month_cell_get_content_space (GcalMonthCell *self)
   gtk_style_context_get_padding (context, &padding);
   gtk_style_context_get_border (context, &border);
 
-  return gtk_widget_get_allocated_height (GTK_WIDGET (self)) -
+  return gtk_widget_get_height (GTK_WIDGET (self)) -
          gcal_month_cell_get_header_height (self) -
          padding.top - padding.bottom -
          border.top - border.bottom;
@@ -525,7 +566,7 @@ gcal_month_cell_get_header_height (GcalMonthCell *self)
   gtk_style_context_get_padding (context, &padding);
   gtk_style_context_get_border (context, &border);
 
-  return gtk_widget_get_allocated_height (self->header_box) +
+  return gtk_widget_get_height (self->header_box) +
          gtk_widget_get_margin_top (self->header_box) +
          gtk_widget_get_margin_bottom (self->header_box) +
          padding.top + padding.bottom +
@@ -537,7 +578,7 @@ gcal_month_cell_get_overflow_height (GcalMonthCell *self)
 {
   g_return_val_if_fail (GCAL_IS_MONTH_CELL (self), -1);
 
-  return gtk_widget_get_allocated_height (self->overflow_button);
+  return gtk_widget_get_height (self->overflow_button);
 }
 
 gboolean
