@@ -257,7 +257,7 @@ on_button_pressed (GtkGestureClick *click_gesture,
   gint width;
 
   ltr = gtk_widget_get_direction (GTK_WIDGET (self)) != GTK_TEXT_DIR_RTL;
-  width = gtk_widget_get_allocated_width (self->scrolledwindow);
+  width = gtk_widget_get_width (self->scrolledwindow);
   column_width = width / 7.0;
   column = ltr ? (x / column_width) : (7  - x / column_width);
 
@@ -282,7 +282,7 @@ on_motion_notify (GtkEventControllerMotion *motion_event,
   gint width;
 
   ltr = gtk_widget_get_direction (GTK_WIDGET (self)) != GTK_TEXT_DIR_RTL;
-  width = gtk_widget_get_allocated_width (self->scrolledwindow);
+  width = gtk_widget_get_width (self->scrolledwindow);
   column_width = width / 7.0;
   column = ltr ? (x / column_width) : (7  - x / column_width);
 
@@ -301,6 +301,7 @@ on_button_released (GtkGestureClick *click_gesture,
   g_autoptr (GDateTime) selection_start = NULL;
   g_autoptr (GDateTime) selection_end = NULL;
   g_autoptr (GDateTime) week_start = NULL;
+  g_autoptr (GcalRange) range = NULL;
   GtkWidget *weekview;
   gboolean ltr;
   gdouble column_width;
@@ -311,7 +312,7 @@ on_button_released (GtkGestureClick *click_gesture,
   gint end;
 
   ltr = gtk_widget_get_direction (GTK_WIDGET (self)) != GTK_TEXT_DIR_RTL;
-  width = gtk_widget_get_allocated_width (self->scrolledwindow);
+  width = gtk_widget_get_width (self->scrolledwindow);
   column_width = width / 7.0;
   column = ltr ? (x / column_width) : (7  - x / column_width);
 
@@ -345,11 +346,12 @@ on_button_released (GtkGestureClick *click_gesture,
   gtk_widget_translate_coordinates (GTK_WIDGET (self),
                                     weekview,
                                     0,
-                                    gtk_widget_get_allocated_height (GTK_WIDGET (self)),
+                                    gtk_widget_get_height (GTK_WIDGET (self)),
                                     NULL,
                                     &out_y);
 
-  gcal_view_create_event (GCAL_VIEW (weekview), selection_start, selection_end, out_x, out_y);
+  range = gcal_range_new (selection_start, selection_end, GCAL_RANGE_DATE_ONLY);
+  gcal_view_create_event (GCAL_VIEW (weekview), range, out_x, out_y);
 
   gtk_event_controller_set_propagation_phase (self->motion_controller, GTK_PHASE_NONE);
 }
@@ -486,6 +488,7 @@ update_overflow (GcalWeekHeader *self)
             }
 
           gtk_label_set_label (GTK_LABEL (label), text);
+          gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
           g_free (text);
         }
       else if (label)
@@ -763,6 +766,9 @@ apply_overflow_at_weekday (GcalWeekHeader *self,
 
   child = gtk_grid_get_child_at (self->grid, weekday, 3);
 
+  if (!child)
+    return;
+
   split_event_widget_at_column (self, child, weekday);
   gtk_widget_set_visible (child, FALSE);
 }
@@ -1106,7 +1112,7 @@ header_expand (GcalWeekHeader *self)
 
   /* TODO: animate this transition */
   gtk_scrolled_window_set_max_content_height (GTK_SCROLLED_WINDOW (self->scrolledwindow),
-                                              gtk_widget_get_allocated_height (week_view) / 2);
+                                              gtk_widget_get_height (week_view) / 2);
 
   gtk_button_set_icon_name (self->expand_button, "go-up-symbolic");
 
@@ -1156,7 +1162,7 @@ get_dnd_cell (GcalWeekHeader *self,
 {
   gdouble column_width;
 
-  column_width = gtk_widget_get_allocated_width (GTK_WIDGET (self)) / 7.0;
+  column_width = gtk_widget_get_width (GTK_WIDGET (self)) / 7.0;
 
   return x / column_width;
 }
@@ -1337,13 +1343,13 @@ gcal_week_header_snapshot (GtkWidget   *widget,
   state = gtk_style_context_get_state (context);
   ltr = gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL;
 
-  start_x = ltr ? gtk_widget_get_allocated_width (self->expand_button_box) : 0;
-  start_y = gtk_widget_get_allocated_height (self->header_labels_box);
+  start_x = ltr ? gtk_widget_get_width (self->expand_button_box) : 0;
+  start_y = gtk_widget_get_height (self->header_labels_box);
 
   gtk_widget_get_allocation (widget, &alloc);
 
   if (!ltr)
-    alloc.width -= gtk_widget_get_allocated_width (self->expand_button_box);
+    alloc.width -= gtk_widget_get_width (self->expand_button_box);
 
   week_start = gcal_date_time_get_start_of_week (self->active_date);
   week_end = g_date_time_add_days (week_start, 6);
