@@ -1124,9 +1124,9 @@ create_dbus_proxy (GDBusConnection *connection,
 }
 
 void
-gcal_utils_launch_online_accounts_panel (GDBusConnection *connection,
-                                         const gchar     *action,
-                                         const gchar     *arg)
+gcal_utils_launch_gnome_settings (GDBusConnection *connection,
+                                  const gchar     *panel_id,
+                                  const gchar     *action)
 {
   g_autoptr (GDBusProxy) proxy = NULL;
   GVariantBuilder builder;
@@ -1135,20 +1135,18 @@ gcal_utils_launch_online_accounts_panel (GDBusConnection *connection,
 
   g_variant_builder_init (&builder, G_VARIANT_TYPE ("av"));
 
-  if (!action && !arg)
+  g_assert (panel_id != NULL && *panel_id != '\0');
+
+  if (!action)
     {
       g_variant_builder_add (&builder, "v", g_variant_new_string (""));
     }
   else
     {
-      if (action)
-        g_variant_builder_add (&builder, "v", g_variant_new_string (action));
-
-      if (arg)
-        g_variant_builder_add (&builder, "v", g_variant_new_string (arg));
+      g_variant_builder_add (&builder, "v", g_variant_new_string (action));
     }
 
-  array[0] = g_variant_new ("v", g_variant_new ("(sav)", "online-accounts", &builder));
+  array[0] = g_variant_new ("v", g_variant_new ("(sav)", panel_id, &builder));
 
   params[0] = g_variant_new_string ("launch-panel");
   params[1] = g_variant_new_array (G_VARIANT_TYPE ("v"), array, 1);
@@ -1162,7 +1160,7 @@ gcal_utils_launch_online_accounts_panel (GDBusConnection *connection,
 
   if (!proxy)
     {
-      g_warning ("Couldn't open Online Accounts panel");
+      g_warning ("Couldn't open panel '%s'", panel_id);
       return;
     }
 
@@ -1260,31 +1258,19 @@ typedef struct
 
 static void
 on_message_dialog_response_cb (GtkDialog         *dialog,
-                               gint               response,
+                               const gchar       *response,
                                AskRecurrenceData *data)
 {
   GcalRecurrenceModType mod_type;
 
-  switch (response)
-    {
-      case GTK_RESPONSE_CANCEL:
-        mod_type = GCAL_RECURRENCE_MOD_NONE;
-        break;
-      case GTK_RESPONSE_ACCEPT:
-        mod_type = GCAL_RECURRENCE_MOD_THIS_ONLY;
-        break;
-      case GTK_RESPONSE_OK:
-        mod_type = GCAL_RECURRENCE_MOD_THIS_AND_FUTURE;
-        break;
-      case GTK_RESPONSE_YES:
-        mod_type = GCAL_RECURRENCE_MOD_ALL;
-        break;
-      default:
-        mod_type = GCAL_RECURRENCE_MOD_NONE;
-        break;
-    }
-
-  gtk_window_destroy (GTK_WINDOW (dialog));
+  if (g_strcmp0 (response, "this-only") == 0)
+    mod_type = GCAL_RECURRENCE_MOD_THIS_ONLY;
+  else if (g_strcmp0 (response, "subsequent-events") == 0)
+    mod_type = GCAL_RECURRENCE_MOD_THIS_AND_FUTURE;
+  else if (g_strcmp0 (response, "all-events") == 0)
+    mod_type = GCAL_RECURRENCE_MOD_ALL;
+  else
+    mod_type = GCAL_RECURRENCE_MOD_NONE;
 
   data->callback (data->event, mod_type, data->user_data);
   g_clear_object (&data->event);
@@ -1299,7 +1285,6 @@ gcal_utils_ask_recurrence_modification_type (GtkWidget                 *parent,
                                              gpointer                   user_data)
 {
   AskRecurrenceData *data;
-  GtkDialogFlags flags;
   ECalClient *client;
   GtkWidget *dialog;
 
@@ -1308,30 +1293,25 @@ gcal_utils_ask_recurrence_modification_type (GtkWidget                 *parent,
   data->callback = callback;
   data->user_data = user_data;
 
-  flags = GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT;
-
-  dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_native (parent)),
-                                   flags,
-                                   GTK_MESSAGE_QUESTION,
-                                   GTK_BUTTONS_NONE,
+  dialog = adw_message_dialog_new (GTK_WINDOW (gtk_widget_get_native (parent)),
+                                   _("Modify Multiple Events?"),
                                    _("The event you are trying to modify is recurring. The changes you have selected should be applied to:"));
 
-  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                          _("_Cancel"),
-                          GTK_RESPONSE_CANCEL,
-                          _("_Only This Event"),
-                          GTK_RESPONSE_ACCEPT,
-                          NULL);
+  adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dialog),
+                                    "close", _("_Cancel"),
+                                    "this-only", _("_Only This Event"),
+                                    NULL);
 
   client = gcal_calendar_get_client (gcal_event_get_calendar (event));
 
   if (!e_client_check_capability (E_CLIENT (client), E_CAL_STATIC_CAPABILITY_NO_THISANDFUTURE))
-    gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Subsequent events"), GTK_RESPONSE_OK);
+    adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "subsequent-events", _("_Subsequent Events"));
 
   if (show_mod_all)
-    gtk_dialog_add_button (GTK_DIALOG (dialog), _("_All events"), GTK_RESPONSE_YES);
+    adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "all-events",  _("_All Events"));
 
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (gtk_widget_get_native (parent)));
+
   g_signal_connect (dialog, "response", G_CALLBACK (on_message_dialog_response_cb), data);
 
   gtk_window_present (GTK_WINDOW (dialog));
